/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2013 Frederik Heber. All rights reserved.
 * Please see the COPYING file or "Copyright notice" in builder.cpp for details.
 *
 *
 *   This file is part of MoleCuilder.
 *
 *    MoleCuilder is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MoleCuilder is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MoleCuilder.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * FourBody_TorsionAngle.cpp
 *
 *  Created on: 03.09.2013
 *      Author: heber
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

//#include "CodePatterns/MemDebug.hpp"

#include "FourBody_TorsionAngle.hpp"

#include "CodePatterns/Assert.hpp"

#include <cmath>

#include "Potentials/helpers.hpp"

double FourBody_TorsionAngle::operator()(const FunctionModel::arguments_t &arguments) const
{
  ASSERT( arguments.size() == 6,
      "FourBody_TorsionAngle() - require exactly six arguments.");
  const double &r_ij = arguments[0].distance; // 01
  const double &r_ik = arguments[1].distance; // 02
  const double &r_il = arguments[2].distance; // 03
  const double &r_jk = arguments[3].distance; // 12
  const double &r_jl = arguments[4].distance; // 13
  const double &r_kl = arguments[5].distance; // 23
  const double h_1 = .5*sqrt(2.*(Helpers::pow(r_ij,2)+Helpers::pow(r_ik,2))-Helpers::pow(r_jk,2));
  const double h_2 = .5*sqrt(2.*(Helpers::pow(r_jl,2)+Helpers::pow(r_kl,2))-Helpers::pow(r_jk,2));
  const double angle = Helpers::pow(h_1,2) + Helpers::pow(h_2,2) - Helpers::pow(r_il,2);
  const double divisor = 2.* h_1 * h_2;

//  LOG(2, "DEBUG: cos(theta)= " << angle/divisor);
  if (divisor == 0.)
    return 0.;
  else
    return angle/divisor;
}
